Package: shinyFiles
Type: Package
Title: A Server-Side File System Viewer for Shiny
Version: 0.9.3
Authors@R: 
    c(person(given = "Thomas Lin",
             family = "Pedersen",
             role = c("cre", "aut"),
             email = "thomasp85@gmail.com",
             comment = c(ORCID = "0000-0002-5147-4711")),
      person(given = "Vincent",
             family = "Nijs",
             role = "aut"),
      person(given = "Thomas",
             family = "Schaffner",
             role = "aut"),
      person(given = "Eric",
             family = "Nantz",
             role = "aut"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: Provides functionality for client-side navigation of
    the server side file system in shiny apps. In case the app is running
    locally this gives the user direct access to the file system without the
    need to "download" files to a temporary location. Both file and folder
    selection as well as file saving is available.
License: GPL (>= 2)
Encoding: UTF-8
Imports: htmltools, jsonlite, tools, shiny (>= 1.1.0), fs (>= 1.2.6),
        tibble (>= 1.4.2)
Collate: 'aaa.R' 'filechoose.R' 'dirchoose.R' 'filesave.R'
        'shinyFiles-package.R'
RoxygenNote: 7.2.1
URL: https://github.com/thomasp85/shinyFiles
BugReports: https://github.com/thomasp85/shinyFiles/issues
Suggests: covr
NeedsCompilation: no
Packaged: 2022-08-19 13:37:35 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut]
    (<https://orcid.org/0000-0002-5147-4711>),
  Vincent Nijs [aut],
  Thomas Schaffner [aut],
  Eric Nantz [aut]
Repository: CRAN
Date/Publication: 2022-08-19 14:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 13:25:46 UTC; unix
